/*******************************************/
/* A call to this function sets up XGrafix */

XGInit(argc, argv, t)
int argc;
char *argv[];
double *t;

/*******************************************************************/
/* This function must be called each time step to capture X events */

XGEvent()

/*******************************************************************/
/* A call to this function each time step updates the OPEN windows */

XGUpdate()

/**********************************************************/
/* The function to set up a 2D window for plotting x vs y */

XGSet2D(PlotType, X_Label, Y_Label, State, ulx, uly, X_Scale, Y_Scale,
	X_Auto_Rescale, Y_Auto_Rescale, X_Min, X_Max, Y_Min, Y_Max)
char  *PlotType;          /* "linlin", "linlog", "loglog", or "loglin"        */
char  *X_Label;           /* x label for the frame                            */
char  *Y_Label;           /* y label for the frame                            */
float  X_Scale;           /* scaling factor for the x array                   */
float  Y_Scale;           /* scaling factor for the y array                   */
char  *State;             /* "open" or "iconic"                               */
int    ulx, uly;          /* requested position of frame's upper left coner   */
int    X_Auto_Rescale;    /* if True X_Min and X_Max are neglected            */
int    Y_Auto_Rescale;    /* if True Y_Min and Y_Max are neglected            */
float  X_Min, X_Max;      /* x bounds for the plot if x-autorescale if False  */
float  Y_Min, Y_Max;      /* y bounds for the plot if y-autorescale if False  */

/**************************************************************************/  
/* The function to set up a 2D window for plotting contours of z vs x & y */

XGSet2DC(PlotType, X_Label, Y_Label, Z_Label, State, ulx, uly,
	 X_Scale, Y_Scale, Z_Scale, X_Auto_Rescale, Y_Auto_Rescale,
	 Z_Auto_Rescale, X_Min, X_Max, Y_Min, Y_Max, Z_Min, Z_Max)
char  *PlotType;          /* "linlinlin", "linlinlog", "linloglog", etc.      */
char  *X_Label;           /* x label for the frame                            */
char  *Y_Label;           /* y label for the frame                            */
char  *Z_Label;           /* z label for the frame                            */
char  *State;             /* "open" or "iconic"                               */
int    ulx, uly;          /* requested position of frame's upper left coner   */
float  X_Scale;           /* scaling factor for the x array                   */
float  Y_Scale;           /* scaling factor for the y array                   */
float  Z_Scale;           /* scaling factor for the z array                   */
int    X_Auto_Rescale;    /* if True X_Min and X_Max are neglected            */
int    Y_Auto_Rescale;    /* if True Y_Min and Y_Max are neglected            */
int    Z_Auto_Rescale;    /* if True Z_Min and Z_Max are neglected            */
float  X_Min, X_Max;      /* x bounds for the plot if x-autorescale if False  */
float  Y_Min, Y_Max;      /* y bounds for the plot if y-autorescale if False  */
float  Z_Min, Z_Max;      /* z bounds for the plot if z-autorescale if False  */

/**************************************************************/
/* The function to set up a 3D window for plotting z vs x & y */

XGSet3D(PlotType, X_Label, Y_Label, Z_Label, Theta, Phi, State, ulx, uly,
	X_Scale, Y_Scale, Z_Scale, X_Auto_Rescale, Y_Auto_Rescale,
	Z_Auto_Rescale, X_Min, X_Max, Y_Min, Y_Max, Z_Min, Z_Max)
char  *PlotType;          /* "linlinlin", "linlinlog", "linloglog", etc.      */
char  *X_Label;           /* x label for the frame                            */
char  *Y_Label;           /* y label for the frame                            */
char  *Z_Label;           /* z label for the frame                            */
float  Theta, Phi;        /* initial viewing angles                           */
char  *State;             /* "open" or "iconic"                               */
int    ulx, uly;          /* requested position of frame's upper left coner   */
float  X_Scale;           /* scaling factor for the x array                   */
float  Y_Scale;           /* scaling factor for the y array                   */
float  Z_Scale;           /* scaling factor for the z array                   */
int    X_Auto_Rescale;    /* if True X_Min and X_Max are neglected            */
int    Y_Auto_Rescale;    /* if True Y_Min and Y_Max are neglected            */
int    Z_Auto_Rescale;    /* if True Z_Min and Z_Max are neglected            */
float  X_Min, X_Max;      /* x bounds for the plot if x-autorescale if False  */
float  Y_Min, Y_Max;      /* y bounds for the plot if y-autorescale if False  */
float  Z_Min, Z_Max;      /* z bounds for the plot if z-autorescale if False  */

/***********************************************************************/  
/* After setting up a 2D window with XGSet2D(), this function can be 
   called to add a curve to the data to be plotted in the window       */

XGCurve(x_array, y_array, npoints, color)
float   *x_array;    /* x_array to be plotted                          */
float   *y_array;    /* y_array to be plotted                          */
int     *npoints;    /* number of points in the x (and y) direction    */
int      color;      /* plot's color chosen from 0-9                   */

/***********************************************************************/  
/* After setting up a 2D window with XGSet2D(), this function can be 
   called to add a 2D scatter plot to the data to be plotted in the window */

XGScat2D(x_array, y_array, npoints, color)
float   *x_array;    /* x_array to be plotted                          */
float   *y_array;    /* y_array to be plotted                          */
int     *npoints;    /* number of points in the x (and y) direction    */
int      color;      /* plot's color chosen from 0-9                   */

/***********************************************************************/  
/* After setting up a 3D window with XGSet3D(), this function can be 
   called to add a 3D scatter plot to the data to be plotted in the window */

XGScat3D(x_array, y_array, z_array, npoints, color)
float   *x_array;    /* x_array to be plotted                          */
float   *y_array;    /* y_array to be plotted                          */
float   *z_array;    /* z_array to be plotted                          */
int     *npoints;    /* number of points in the x, y and z direction   */
int      color;      /* plot's color chosen from 0-9                   */

/***********************************************************************/  
/* After setting up a 3D window with XGSet3D(), this function can be 
   called to add a surface plot to the data to be plotted in the window*/

XGSurf(x_array, y_array, z_array, mpoints, npoints, color)
float   *x_array;    /* x_array to be plotted                          */
float   *y_array;    /* y_array to be plotted                          */
float  **z_array;    /* z_array to be plotted                          */
int     *mpoints;    /* number of points in the x direction            */
int     *npoints;    /* number of points in the y direction            */
int      color;      /* plot's color chosen from 0-9                   */

/***********************************************************************/  
/* After setting up a 2D window with XGSet2DC(), this function can be 
   called to add a contour plot to the data to be plotted in the window*/

XGCont(x_array, y_array, z_array, mpoints, npoints, color)
float   *x_array;    /* x_array to be plotted                          */
float   *y_array;    /* y_array to be plotted                          */
float  **z_array;    /* z_array to be plotted                          */
int     *mpoints;    /* number of points in the x direction            */
int     *npoints;    /* number of points in the y direction            */
int      color;      /* plot's color chosen from 0-9                   */

/***********************************************************************/  
/* XGRead() reads into a block pointed to by 'ptr', 'nitems' items of
   data from the named input stream 'stream', where an item of data is
   a sequence of bytes (not necessarily terminated by a null byte) of
   length 'size'. 'type' is either "float", "int", or "char".          */  

XGRead(ptr, size, nitems, stream, type)
char *ptr, *type;
FILE *stream;
int size, nitems;

/**********************************************************************/
/* XGWrite() writes at most 'nitems' items of data of size 'size' from
   the block pointed to by 'ptr' to the named output stream 'stream'.
   'type' is either "float", "int", or "char".                        */  

XGWrite(ptr, size, nitems, stream, type)
char *ptr, *type;
FILE *stream;
int size, nitems;

/**********************************************************************/
