#include "ibc.h"
#define min(A,B)	((A) < (B) ? (A) : (B))

circuit()
{
  register int i, m;	
  float temp1, temp2, temp3;
  
  /*****************************/
  /* Input voltage at time `t' */
  Vin  = Vo*min(t/lambda,1.0)*sin(t*TWOPI/lambda);
  
  /* Change the circuit voltage at time `t' */
  if (driver == 0)   /* dRHO/dt is the driving term */
    {
      temp1 = BCcoup*Qb0;
      for (i=0; i<nc-n1; i++)
	Vc[i] += (Ic[i] - Ic[i+1] + temp1*FDr[i] )/C0;
      temp2 = (4.0*(n1+1));
      for (i=nc-n1; i<=nc; i++)
	{
	  temp3 = cos(TWOPI*(i - nc + n1)/temp2)*temp1*FDr[i];
	  Vc[i] += (Ic[i] - Ic[i+1] + temp3)/C0;
	}
    }
  else               /* dIb/dx is the driving term */
    {
      temp1 = BCcoup*Ib0;
      for (i=0; i<nc-n1; i++)
	Vc[i] += (Ic[i] - Ic[i+1]+temp1*FDr[i] )/C0;
      temp2 = (4.0*(n1+1));
      for (i=nc-n1; i<=nc; i++)
	{
	  temp3 = cos(TWOPI*(i - nc + n1)/temp2)*temp1*FDr[i];
	  Vc[i] += (Ic[i] - Ic[i+1] + temp3)/C0;
	}
    }
  
  /*****************************************/
  /* Smoothing routine for Vc...           */ 
  
  for (m=0; m<NsmoothVc; m++)
    {
      x = Vc[0];
      
      /* First, use the `1 2 1' digital filter */
      for (i=0; i<nc; i++)	Vc[i] += Vc[i+1];
      
      for (i=nc-1; i>0; i--)	Vc[i] += Vc[i-1];
      
      for (i=1; i<nc; i++)	Vc[i] /= 4;
      
      Vc[0] = x;						  	  
      
      /* Second, use the `-1 6 -1' compensator */
      for (i=0; i<nc; i++)	Vc[i] -= 5.8284271*Vc[i+1];
      
      for (i=nc-1; i>0; i--)	Vc[i] -= 5.8284271*Vc[i-1];
      temp1 = 4*5.8284271;
      for (i=1; i<nc; i++)	Vc[i] /= temp1;
      
      Vc[0] = x;
    }											  
  
  /*************************************************************/
  /* Change in the circuit current at time `time+1/2'          */
  /* Also calculate the electric field within each pitch length*/
  
  for (i=0; i<nc+2; i++)
    Iold[i]  = Ic[i];
  
  Ec[0] = Vin - Vc[0];
  Ic[0] = ( 2*Ec[0] - (Rin - L0)*Ic[0] )/(Rin + L0);
  for (i=1; i<= nc; i++)
    {
      Ec[i] = Vc[i-1] - Vc[i];
      Ic[i] = ( Ec[i] - (0.5*Rc[i]-L0)*Ic[i] )/(0.5*Rc[i]+L0);
    }
  Ic[nc+1] = (2*(Vc[nc] - Vout) - (Rout - L0)*Ic[nc+1])/(Rout + L0);
}
