#include "ibc.h"

drive()
{
  register int i, m, temp1;
  float temp[1030];
  
  /*****************************************************/
  /* Choose the driving term, either dRHO/dt or dIb/dx */
  
  if (driver == 0)      /* dRHO/dt is the driving term */
    for (i=0; i<= nc; i++)	Dr[i] = Qn[i] - Qo[i];
  
  else			 /* dIb/dx is the driving term  */
    for (i=0; i<= nc; i++)	Dr[i] = Ib[i+1] - Ib[i];
  
  
  /*****************************************************/
  /* Use either a Fourier filter or a Digital filter	  */
  /*	for the driving term 								  	  */
  
  if (filter == 0)      /* Use Fourier filter */
    {			/* Initialize Re and Im for FFT w/ nfft elements*/
      
      for (i=0; i< ns; i++) Im[i] = Re[i] = 0;
      
      for (i=ns; i<= ns+nc; i++)
	{
	  Im[i] = 0.0;
	  k     = i - ns;
	  Re[i] = Dr[k]-Dr[0];
	}
      temp1 = 2*ns+2*nc;
      for (i=ns+nc+1; i<= nfft; i++)
	{
	  Im[i] = 0.0;
	  Re[i] = Re[temp1-i]; 
	}
      
      /* fast fourier transform */
      /* first, add one to index because of indices used in fft */
      
      for (i=nfft+1; i>0; i--)	Re[i] = Re[i-1]/nfft;
      
      cpft(Re, Im, nfft, 1, 1.);		/* fast fourier transform */	 
      
      for (i=1; i<= nfft/2; i++)	 /* filter K spectrum	*/
	{
	  x = i/Kmax;		
	  x *= x;
	  x *= x;
	  temp[i] = 1/(1 + x);
	}
      temp1 = nfft+2;
      for (i=2; i<= nfft/2; i++)
	{
	  Re[i] *= temp[i-1];
	  Im[i] *= temp[i-1];
	  k      = temp1-i;
	  Re[k] *= temp[i-1];
	  Im[k] *= temp[i-1];
	}
      i = nfft/2 + 1;
      Re[i] *= temp[i-1];
      Im[i] *= temp[i-1];
      
      cpft(Re, Im, nfft, 1, -1.);	/* inverse fast fourier transform */
      
      /**********************************************************/
      /* finally, subtract one from index to return to original */
      /*	indices, taking only the relevant nc elements.			 */
      
      for (i=0; i<= nc; i++)	
	FDr[i] = Re[i+ns+1];
    }
  else    				 /* Use digital filter */
    {	       				 /* repeat digital filter `NsmoothDr' times */ 
      Dr[nc+1]=0;
      for (i=0; i<nc+2; i++)	FDr[i] = Dr[i];
      
      for (m=0; m<NsmoothDr; m++)
	{
	  /*****************************************/
	  /* First, use the `1 2 1' digital filter */
	  
	  x = FDr[0];
	  for (i=0; i<= nc; i++)	FDr[i] += FDr[i+1];
	  
	  for (i=nc; i>0; i--)		FDr[i] += FDr[i-1];
	  
	  for (i=1; i<= nc; i++)	FDr[i] /= 4;
	  
	  FDr[0]  = x;
	  
	  /*************************************************/
	  /* Second, use the `-1 6 -1' digital compensator */
	  
	  for (i=0; i<= nc; i++)	FDr[i] -= 5.8284271*FDr[i+1];
	  
	  for (i=nc; i>0; i--)		FDr[i] -= 5.8284271*FDr[i-1];
	  temp1 = 4*5.8284271;
	  for (i=1; i<= nc; i++)	FDr[i] /= temp1;
	  
	  FDr[0] = x;
	}
    }
}
