/************************************************************************
**
**	IBC - INTERACTIVE BEAM-CIRCUIT SIMULATION	CODE
**	
**	REVISION/PROGRAMMER/DATE
**	
**	1.0 Original C code /Ian J. Morey /1988
**
**	Conversion to WinGraphics (version 2.0) interface 
**          /Edward H. Chao and Vahid Vahedi/07-10-90
**	
**      Conversion to XGRAFIX interface by Edward H. Chao 1991
**               XGRAFIX interface by /Vahid Vahedi and John Verboncoeur/
**
**	1.1 Addition of Collector Energy Diagnostic and Power Diagnostic
**		 in real units (Watts). /Edward H. Chao 1992/
**
** 	This is a first attempt at writing a Traveling-Wave-Tube code. It 
**	uses the equations descibed by Brillouin (1949) and includes the effects 
**	of space charge similar to the method described by Hess (1961).
**
**	
**				INPUT PARAMETERS
**
** Length = Length of the circuit               ...( Meters)
** Vbeam  = Beam velocity                       ...( Meters/second)
** Ibeam  = Beam current                        ...( Amps )
** Rbeam  = Beam radius	                        ...( Meters )
** Vphase = Phase velocity of the circuit wave  ...( Meters/second)
** Vin    = Input voltage                       ...( Volts)
** Freq   = Frequency of the input voltage      ...( Cycles/second)
** Zo     = Circuit impedance                   ...( Ohms )
** Lo     = Circuit inductance/meter            ...( Henrys/meter )	
** Co     = Circuit capacitance/meter           ...( Farads/meter)
**
**				CODE PARAMETERS
**
** nc     = number of cells
** dx     = Cell width                          ...( Meters)
** dt     = Time step 	 		        ...( Seconds)
** vb     = beam velocity in units of cell widths per time step
** ne     = standard number of particles per cell
** nb     = number of beam electrons injected per time step, nb = vb*ne
** Vb 	 = Voltage at the beam
** Ib 	 = Beam Current/Injected beam current
** Rb 	 = Beam Radius in units of cell width
** Vc 	 = Circuit Voltage
** Ic 	 = Circuit Current
** Q     = Beam charge/length: Qo, Qn = old, new values respectively
** pos   = array containing the position of the beam electrons 
** vel   = array containing the velocity of the beam electrons 
**
***************************************************************************/

#define GLOBALORIGIN
#include "ibc.h"
#undef GLOBALORIGIN

/********************************************************************/

main(argc, argv) 
int argc;
char *argv[];
{
  int msg;
  
  display_title();
  init(argc, argv);		
  InitWindows(argc, argv);		/* Initialize Windows */
  
  /***********************************/
  /* Start of loop to evolve in time */
  
  while(True)
    {
      XGEvent();

      t++;		/* Increment the time */
	  
      move();
      drive();
      circuit();
      field();
      history();

      XGUpdate();
    }
}

/*********************** End of MAIN ***************************/


/***************************************************************/

history()
{
  int i;

  for (i=0; i < nc; i++)
    {
      Qn_diag[i] = 1 - Qn[i];
      Es_diag[i] = phi[i] - phi[i+1];
      
      power[i] = Vc[i]*Iold[i] + DBL_MIN;
    }
  Qn_diag[nc] = 1 - Qn[nc];
  Es_diag[nc]= Es_diag[nc-1];
  
  power[nc] = Vc[nc]*Iold[nc] + DBL_MIN;
}

/***************************************************************/

display_title()
{
  puts("\n\nIBC - Interactive Beam-Circuit Simulation Code");
  puts("version 1.1");
  puts("(c) Copyright 1988-91 Regents of the University of California");
  puts("Plasma Theory and Simulation Group");
  puts("University of California - Berkeley");
}
