/*************Initialization****************************/
#include "ibc.h"


init(argc, argv)
int argc;
char *argv[];
{
  register int i;
  FILE *fp;
  int  flag10=0;
  char a_char[80];
  float prc_Vo, prc_C, prc_b, prc_R, prc_n, prc_wq, prc_QC;
  
  /***************************************************/
  /* Default values of circuit and beam parameters */
  
  Length	= 1e-1;
  Vbeam		= 1e+7;
  Ibeam		= 1e-3;
  Rbeam		= 1e-2;
  Freq		= 5e+8;
  Vin		= 1e-2;
  Vphase	= 1e+7;
  Zo		= 2e+1;						  
  Rin		= 2e+1;						  
  Rout		= 2e+1;	
  Ro  		= 1.1513e-2;	
  gamma_func	= 0.1;	
  
  /***************************************************/
  /* Load default number of cells and particles */
  
  nc     	= 100;
  ne    	= 2;
  n0     	= 40;
  n1    	= 10;
  
  /*********************************************************************/
  /* Load default values of the Beam-Circuit and Space-Charge coupling */
  
  BCcoup 	= 1.0;
  SCcoup 	= 1e-6;
  
  /************************************************************************/
  /*Load default driving term and filter, dRHO/dt and K-space respectively*/
  
  driver  = 1;
  filter  = 0;
  
  /*********************************************/
  /* Load default /kmax in K-space filter and 
     number of loops in the digital filter */
  
  kmax		= 2;
  NsmoothDr	= 10;
  NsmoothVc	= 1;
  
  /****************************************************************/
  /* Read new values from the input file; default is `ibc.inp' */
  
  if (argc<2)
    {
      fp = fopen("ibc.inp","r");
      flag10= 1;
    }
  else 
    fp = fopen(argv[1],"r");
  
  if ((argc>1) && !fp){
    for (i=0; argv[1][i] != 0; i++) a_char[i] = argv[1][i];
    a_char[i++] = '.';		/*try to add .inp */
    a_char[i++] = 'i';
    a_char[i++] = 'n';
    a_char[i++] = 'p';
    a_char[i++] = 0;
    fp = fopen(a_char,"r");
  }
  
  if (!fp){
    if(flag10)
      printf("\nCan't find IBC.INP.");
    else
      printf("\nCan't find input file '%s'.",argv[1]);
    printf(" The default parameters will be used.\n");
  }
  else{
    while (fscanf(fp, " %e %e %e ", &Length, &Vbeam, &Vphase) <3)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %e %e %e ", &Ibeam, &Rbeam, &Freq) <3)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %e %e %e %e ", &Zo, &Rin, &Rout, &Vin) <4)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %e %d %e %d ", &Ro, &n0, &gamma_func, &n1) <4)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %d %d %g %g %d ", &nc, &ne, &emin, &emax, &nbins) <5)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %f %f ", &BCcoup, &SCcoup) <2)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %d %d ", &driver, &filter) <2)
      fscanf(fp, " %s ", a_char);
    
    while (fscanf(fp, " %f %d %d ", &kmax, &NsmoothDr, &NsmoothVc) <3)
      fscanf(fp, " %s ", a_char);
  }
  fclose(fp);
  
  /*****************************************************************/
  /* The radial variation of the AC beam density is assumed to be
     proportional to the zero Bessel function out to the first 
     zero. So Ne(r,t) = N0 + N1*exp(j[kx-wt])*Jo(2.405*r/Rbeam), 
     r < Rbeam. Therefore,  */
  
  No = Ibeam*1.987e18/Vbeam/Rbeam/Rbeam;
  Wo = 8.9814*sqrt(fabs(No));
  Co	= 1.0/(Zo*Vphase);
  Lo	= Zo/Vphase;
  
  /****************************************************************/
  /* Initialize FFT parameters: the number of points, nfft = 2**N */
  
  nfft = 2;
  while ( 2*nc > nfft ) nfft *= 2;
  ns = nfft/2 - nc;
  /* 	csdri = "dRHO/dt";
	csfil = "fourier";  */ 
  
  printf("\n\t\t\tTWT PARAMETERS IN MKS UNITS \n\n");
  
  printf("Length    = %8.2e   Beam Velocity = %8.2e   Wave Velocity = %8.2e \n\n",
	 Length, Vbeam, Vphase);
  
  printf("Frequency = %8.2e   Beam Current  = %8.2e   Beam Radius   = %8.2e \n\n", 
	 Freq, Ibeam, Rbeam);
  
  printf("Zo = %8.2e    Rin = %8.2e     Rout = %8.2e      Vin = %8.2e \n\n",
	 Zo, Rin, Rout, Vin);
  
  printf("Ro = %8.2e     n0 = %3d          gamma_func = %8.2e       n1 = %3d \n\n", 
	 Ro, n0, gamma_func, n1);
  
  printf("Lo = %8.2e     Co = %8.2e       No = %8.2e       Wo = %8.2e \n\n",
	 Lo, Co, No, Wo);
  
  printf("WaveLength = %8.2e     Mode = %6.3f \n\n", 
	 Vphase/Freq, Length*Freq/Vphase);
  
  printf("kmax/ko = %8.2e      NsmoothDr = %2d      NsmoothVc = %2d \n\n",
	 kmax, NsmoothDr, NsmoothVc);
  
  printf("nc = %3d     nfft = %3d     ne = %3d\n\n",
	 nc, nfft, ne);
  
  printf("emin = %g	emax = %g	nbins = %d\n\n", emin, emax, nbins);

  printf("BCcoup = %8.2e   SCcoup = %8.2e \n\n",
	 BCcoup, SCcoup);
  
  printf("driver = %d",driver);
  if (driver == 0)	printf("\t(dRHO/dt)\n");
  else printf("\t(dIb/dx)\n");
  
  printf("filter = %d",filter);
  if (filter == 0) printf("\t(Fourier filter)\n");
  else printf("\t(digital filter)\n");

/********** Translating IBC variables to Pierce variables ************************/

  prc_Vo = 0.5*Vbeam*Vbeam/QeMe;
  prc_C  = pow((Zo*Ibeam/prc_Vo/4),1.0/3.0);
  prc_b  = (Vbeam/Vphase - 1)/prc_C;
  prc_R  = sqrt(1/(1+(2.405*Vbeam/Rbeam/Freq/TWOPI)*(2.405*Vbeam/Rbeam/Freq/TWOPI)));
  prc_n	 = Ibeam/Rbeam/Rbeam/PI/1.609e-19/Vbeam;
  prc_wq = prc_R*TWOPI*9*sqrt(prc_n);
  prc_QC = (prc_wq/Freq/TWOPI/(1+prc_wq/Freq/TWOPI))
    *(prc_wq/Freq/TWOPI/(1+prc_wq/Freq/TWOPI))/4/prc_C/prc_C;
  
  printf("\n************ Pierce Variables *****************\n");
  printf("\n(Growth Factor): C = %f\n",prc_C);
  printf("\t\t b = %f\n",prc_b);
  printf("\t\t R = %f\n",prc_R);
  printf("\t\t wq= %g\n",prc_wq);
  printf("\t\t QC= %f\n",prc_QC);

  /************************************/
  /* Print fatal error messages */
  
  if (Length <= 0)     					
    printf("Length is invalid, must exit\n");
  
  if (Vbeam  <= 0 || 3e+8 <= Vbeam )  
    printf("Vbeam is invalid, must exit\n");
  
  if (Vphase <= 0 || 3e+8 <= Vphase)  
    printf("Vphase is invalid, must exit\n");
  
  if (Freq	  <= 0)		 					
    printf("Frequency is invalid, must exit\n");
  
  if (Ibeam  <= 0) 							
    printf("Ibeam is invalid, must exit\n");
  
  if (Rbeam  <= 0) 							
    printf("Rbeam is invalid, must exit\n");
  
  if (Zo	  <= 0)
    printf("Zo is invalid, must exit\n");
  
  if (Rin	  <= 0)
    printf("Rin is invalid, must exit\n");
  
  if (Rout	  <= 0)
    printf("Rout is invalid, must exit\n");
  
  if (ne	  <= 0)
    printf("The average number of electrons per cell is invalid\n");
  
  if (n0 < 0 || 2*n0 > nc)
    printf("n0 is not within acceptable range\n");
  
  if (n1 < 0 || n1 > nc)
    printf("n1 is not within acceptable range\n");
  
  if (nc     >  NCMAX-3)
    printf("The number of cells is too large, must exit\n");
  
  if (nc*ne  >= NPMAX)
    printf("The number of particles is too large,must exit\n");
  
  /******************************/
  /* Print warning messages */
  
  if (Ibeam/(Vbeam/5.93e5)/(Vbeam/5.93e5)/(Vbeam/5.93e5) > 1e-5)
    printf("Perveance > 1e-5\n");
  if (Vbeam/Vphase <= 0.75)	printf("Vbeam is much less than Vphase\n");
  if (Vbeam/Vphase >= 1.25)	printf("Vbeam is much greater than Vphase\n");
  if (Zo 	 != Rin	 )      printf("Input not matched\n");
  if (Zo 	 != Rout )	printf("Output not matched\n");
  if (Ro*Length	 >= Zo/10)	printf("Circuit resistance is large\n");
  
  if (Vphase*nc/Freq/Length < 19.9999) 
    printf("The wavelength is less than 20 cells => reduced accuracy.\n");
  
  if (driver != 0 && driver != 1)
    {
      driver = 0;
      printf("Driver input was invalid, has been set to dRHO/dt \n");
    }
  
  if (filter != 0 && filter != 1)
    {
      filter = 0;
      printf("Filter input was invalid, has been set to fourier \n");
    }
  
  if (n0 < n1)
    {
      n0 = n1;
      printf("n0 was less than n1, but has now been set equal to n1 \n");
    }	
  
  if (kmax	<= 0)
    {
      kmax = 2;
      printf("kmax has been set to the default value %8.3e \n", kmax);
    }
  
  /*********************/
  /* Rescale variables */
  
  ng 	= nc+1;
  np 	= ne*nc;
  dx	= Length/nc;
  dt	= dx/Vphase;                    /* dt defined as dx/Vphase */
  vb	= Vbeam/Vphase;
  nb	= ne*vb;
  Rb	= Rbeam/dx;
  C0    = Co/Ep0; 
  L0    = 1.0/C0;
  Rin   = Rin*L0/Zo;
  Rout  = Rout*L0/Zo;
  Ro	= Ro*dx*L0/Zo;
  Vo    = Vin*(QeMe/Vphase/Vphase);
  Vout  = 0.0;
  Ib0	= Ibeam*(QeMe/Ep0/Vphase/Vphase/Vphase);
  Qb0	= Ibeam*(QeMe/Ep0/Vphase/Vphase/Vbeam);
  lambda= Vphase/Freq/dx;
  Kmax	= kmax*(Length*Freq/Vbeam)*(nfft/nc);
  Escale= .5*Vphase*Vphase/QeMe;
  energy_interval = (emax - emin)/(nbins-1);

  /************************************************/
  
  printf("\n\n     INPUT PARAMETERS IN CODE UNITS \n\n");
  
  printf("dx  = %8.2e   dt = %8.2e     vb = %8.2e          Vo = %8.2e \n\n", 
	 dx, dt, vb, Vo);	
  
  printf("C0  = %8.2e   L0 = %8.2e    Rin = %8.2e        Rout = %8.2e \n\n",
	 C0, L0, Rin, Rout);
  
  printf("Qb0 = %8.2e  Ib0 = %8.2e  Wo*dt = %8.2e  Wavelength = %8.4f \n\n",
	 Qb0, Ib0, Wo*dt, lambda);
  
  printf("nb  = %2.4f \n\n", nb);	
  
  /****************************************************************/
  /* Allocating memory for the variables.  */

  x_array= (float *)malloc(ng*sizeof(float));
  for (i=0; i <= nc; i++) x_array[i] = i*dx;
  
  Ec_diag   = (float *)malloc(ng*sizeof(float));
  Es_diag   = (float *)malloc(ng*sizeof(float));
  Qn_diag   = (float *)malloc(ng*sizeof(float));
  power	    = (float *)malloc(ng*sizeof(float));

  fe	= (float *)malloc(nbins*sizeof(float));
  e_array = (float *)malloc(nbins*sizeof(float));

  Vc  = (float *)malloc(ng*sizeof(float));
  phi = (float *)malloc((ng+1)*sizeof(float));
  Dr  = (float *)malloc((ng+1)*sizeof(float));
  FDr = (float *)malloc((ng+1)*sizeof(float));
  Ib  = (float *)malloc((ng+1)*sizeof(float));
  Ic  = (float *)malloc((ng+1)*sizeof(float));
  Iold= (float *)malloc((ng+1)*sizeof(float));
  Rc  = (float *)malloc(ng*sizeof(float));
  Qn  = (float *)malloc(ng*sizeof(float));
  Qo  = (float *)malloc(ng*sizeof(float));
  Ec  = (float *)malloc((ng+1)*sizeof(float));
  xp  = (float *)malloc(ng*sizeof(float));
  yp  = (float *)malloc(ng*sizeof(float));
  
  pos  = (float *)malloc((NPMAX+1) *sizeof(float));
  vel  = (float *)malloc((NPMAX+1) *sizeof(float));
  Eb   = (float *)malloc((NPMAX+1) *sizeof(float));

  /******************************/
  /* Initialize other variables */
  
  for (i=0; i<ng; i++)
    {
      Vc[i] = phi[i] = Dr[i] = FDr[i] = Ib[i] = Ic[i] = Iold[i] = Rc[i] = 0.0;
      Qn[i] = Qo[i] = Ec[i] = xp[i] = yp[i] = 0.0;
      Ec_diag[i] = Es_diag[i] = Qn_diag[i] = power[i] = 0.0;
    }
  phi[ng] = Dr[ng] = FDr[ng] = Ib[ng] = Ic[ng] = Iold[ng] = 0.0;
  Ec[ng] = 0.0;
  
  Vin = Vo;
  for (i=0; i<nbins; i++)	fe[i] = 0.0;
  for (i=0; i<= np; i++)	pos[i] = vel[i] = Eb[i] = 0.0;
  
  for(i=n0; i<=nc-n0; i++)
    Rc[i] = Ro*( 1 - cos(TWOPI*(i - n0 + 1)/(nc - 2*n0 + 2)) );
  
  Rc[nc-n1] = 1e-6;
  
  for(i=nc-n1+1; i<=nc; i++)	Rc[i] = (1 + gamma_func)*Rc[i-1];
  
  for(i=nc-n1; i<=nc; i++)	Rc[i] -= 1e-6;
  for (i=0; i<nbins; i++)
    e_array[i] = emin + i*energy_interval;

  /**************************************************************/
  /* Load the beam electrons.  Start from a dump file if given, */
  /* otherwise load the system with uniform electrons.		*/

  if (argc > 2)
    {
      strcpy(a_char, argv[2]);
      if(access(a_char, F_OK)) strcat(a_char, ".dmp");
    }
  if(argc > 2 && !access(a_char, F_OK)) Restore(a_char);
  else
    {
      for (i=0; i<np ; ++i)
	{
	  pos[i] = (0.5 + i)/ne;
	  vel[i] = vb;
	}
      adde	 = 1 - .5/vb/ne;
    }
}
